@@ -1,6 +1,6 @@
 <?PHP
 /**
- * Copyright 2010-2014 Horde LLC (http://www.horde.org/)
+ * Copyright 2010-2015 Horde LLC (http://www.horde.org/)
  * Copyright (c) 2010 Gerd Schaufelberger
  * All rights reserved.
  *
@@ -30,7 +30,7 @@
  * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
  * @category  Horde
- * @copyright 2010-2014 Horde LLC
+ * @copyright 2010-2015 Horde LLC
  * @copyright 2010 Gerd Schaufelberger
  * @license   http://www.horde.org/licenses/bsd New BSD License
  * @package   Mail
@@ -39,13 +39,14 @@
 /**
  * SMTP MX implementation.
  *
- * @author    Gerd Schaufelberger <gerd@php-tools.net>
- * @author    Michael Slusarz <slusarz@horde.org>
- * @category  Horde
- * @copyright 2010-2014 Horde LLC
- * @copyright 2010 Gerd Schaufelberger
- * @license   http://www.horde.org/licenses/bsd New BSD License
- * @package   Mail
+ * @author     Gerd Schaufelberger <gerd@php-tools.net>
+ * @author     Michael Slusarz <slusarz@horde.org>
+ * @category   Horde
+ * @copyright  2010-2015 Horde LLC
+ * @copyright  2010 Gerd Schaufelberger
+ * @deprecated Use Horde_Mail_Transport_Hordesmtp instead
+ * @license    http://www.horde.org/licenses/bsd New BSD License
+ * @package    Mail
  */
 class Horde_Mail_Transport_Smtpmx extends Horde_Mail_Transport
 {
@@ -348,7 +349,7 @@ class Horde_Mail_Transport_Smtpmx extends Horde_Mail_Transport
             $replace = $search = array();
 
             foreach ($info as $key => $value) {
-                $search[] = '{' . strtoupper($key) . '}';
+                $search[] = '{' . Horde_String::upper($key) . '}';
                 $replace[] = $value;
             }
 
